IannixObject {
	var <type, <parent, <id, <group, <color, <action;
	var activity, position, autosize, <size, lineStyle, messages, colors, textures; //debug: not used by now
	var <netAddress;
//	var waitList,spool; //...
	
	*preNew {|type, parent, id|
		^super.newCopyArgs(type, parent, id).xinit(parent, type, id)
	}
	
	xinit {|p, t, i|
		var family;
		netAddress = p.iannixAddress;
		family = p.objects[(t++"s").asSymbol];
		if (i.isNil) 
			{
				if (family.isEmpty.not)
					{i = family.keys.asArray.sort.last+1 }
					{i = parent.idOffsets[(t++"s").asSymbol]}
			}; //automatic indexing
		id = i.asInteger; 
		p.objects[(t++"s").asSymbol][id] = this;
		/* //...
			waitList = parent.waitList;
			spool = parent.spool;
		*/
	
	}
	
	send {|autoID=false|
		var idArg;
		if (autoID.not){idArg = id}{idArg = "auto"};
		netAddress.sendMsg("/iannix/add" +  type, idArg);
		this.messages_(\id); //default setting of output messages in IanniX
		^this;
	} 
	
	position_ {|x=0, y=0, z=0|
		netAddress.sendMsg("/iannix/setPos", id, x.asFloat, y.asFloat, z.asFloat)
		^this;
	}
	
	group_ { |groupName|
		netAddress.sendMsg("/iannix/setGroup", id, groupName)
		^this;
	}
	
	active {|bool=true|
		netAddress.sendMsg("/iannix/setActive", id, if (bool){1}{0} )
		^this;
	}

	autosize {|bool=true|
		netAddress.sendMsg("/iannix/autosize", id, if (bool){1}{0} )
		^this;
	}

	
	size_ {|size|
		netAddress.sendMsg("/iannix/setSize", id, size)
		^this;
	}	

	
	thickness_ {|thickness=1|
		netAddress.sendMsg("/iannix/setThickness", id, thickness)
		^this;
	}
	
	action_ {|function|
		action = function;
		^this;
	}
	
	color_ {|colorObject, state=\active|
		state = state.asString;
		state = state.first.toUpper++state[1..];
			
		netAddress.sendMsg("/iannix/setColor"++state, id, 
			(colorObject.red*255).asInteger, (colorObject.green*255).asInteger, (colorObject.blue*255).asInteger, (colorObject.alpha*255).asInteger
		);
		^this;
	}
	
	lineStyle_{| pattern=0xFFFF, factor=1 | 
		//pattern as openGl patterns (16bit hex, or binary, like 2r1111110011110000)
		// http://fly.cc.fer.hr/~unreal/theredbook/chapter02.html, look at "Stippled Lines"
		netAddress.sendMsg("/iannix/setLine", id, pattern, factor)
	}
		
	
	remove {  //debug: object destruction is not done
		netAddress.sendMsg("/iannix/remove", id);		
		parent.objects[(type++"s").asSymbol].removeAt(id);
	}
	
	messages_ {|messages, protocol="osc", ip="127.0.0.1", addr=57120|
		var str = ","+ protocol++":\/\/"++ip++":"++addr++"\/"++type+ type.asString++"_id"; //first mendatory element in the message 
		if ((messages.isNil) || (messages == \id)) {messages = [""]};
		messages.do {|val| str = str + val.asString };
		
		netAddress.sendMsg("/iannix/setMessage", id, parent.messagePeriods[type], str);
		^this;
	}
	
/* to be continued...
setLine
other colors types, textures...
*/
}






IannixTrigger : IannixObject {
	var <x, <y, <z, <value;
	var <action;

	*new {|parent, id|
		^super.preNew(\trigger, parent, id)
	}
	


	triggerOff_ {|delay|
		netAddress.sendMsg("/iannix/setTriggerOff", id, delay);
		^this;
	}

}
	

	
IannixCursor : IannixObject {

	*new {|parent, id|
		^super.preNew(\cursor, parent, id);
	}
	
	putOnCurve {|curve|
		netAddress.sendMsg("/iannix/setCurve", id , curve.id);
		^this;	
	}
	
	width_ {|width=0.1|
		netAddress.sendMsg("/iannix/setWidth", id,  width);
		^this;	
	}
	
	speed_ {|factor=1|
		netAddress.sendMsg("/iannix/setSpeed", id, factor);
		^this;
	}
	
	duration_ {|duration=1|
		netAddress.sendMsg("/iannix/setSpeed", id, "auto", duration);
		^this;
	}
	
	masterSpeed {|factor=1| //? seems like setSpeed to me... 
		netAddress.sendMsg("/iannix/setSpeedF", id, factor);
		^this;
	}
	
	loopPattern_ {|list| //array of speeds, negatives ones for reverse, 0 for stop
		var pattern = "";
		list ?? {list = [1]};
		list.do { |item| pattern = pattern + item };
		netAddress.sendMsg("/iannix/setPattern", id, 0, 0, pattern);
		^this;
	}
	
	offset_ {|initialPoint=0, startPoint=0, endPoint="end"| //endPoint="end", if you don't know the length of the associated curve 
		netAddress.sendMsg("/iannix/setOffset", id, initialPoint, startPoint, endPoint);
		^this;
	}
	
	timePos_ {|pos=0| //in seconds
		netAddress.sendMsg("/iannix/setTime", id, pos);
		^this;
		
	}
	
	bounds_ {|array | //four values: x-top-left-corner, y-top-left-corner, x-bottom-right-corner, y-bottom-right-corner
		var str = "";
		array.do {|val| str = str+val };
		netAddress.sendMsg("/iannix/setBoundsSource", id, str);
		^this;
	}
	
	mapping_ {|array | //four values idem
		var str = "";
		array.do {|val| str = str+val };
		netAddress.sendMsg("/iannix/setBoundsTarget", id, str);
		^this;
	}
		

	
}

IannixCurve : IannixObject {
	var shape;
	
	*new {|parent, id|
		^super.preNew(\curve, parent, id)
	}
	
	points_ {|pointsArray, beziers| //bezier: 2 values = quad, 4 values = cubic (see iannix doc)
		beziers ?? {beziers = ((""!2)!(pointsArray.size)).flat};
		pointsArray.do {|pt, i|
			var bzStr = "";
			beziers[i].do {|item| bzStr = bzStr + item };
			netAddress.sendMsg("/iannix/setPointAt", id, i, pt.x, pt.y, bzStr );
		};
		^this;	
			
		
	}
	
	fromSVGpath {|svgPath, width, height| 
		netAddress.sendMsg("/iannix/setPointAt", id, width, height, svgPath);
		^this;
	}//to be tested
	
	asEllipse {|width, height|
		netAddress.sendMsg("/iannix/setPointsEllipse", id, width, height);
		^this;
	}

	asGlyph{|text,  scale=0.1, fontFamily="Arial" | 
		netAddress.sendMsg("/iannix/setPointsTxt", id, scale, fontFamily, text);
		^this;
	}

	asImage {	|imagePath, scale=1 |
		netAddress.sendMsg("/iannix/setPointsImg", id, scale, imagePath);
		^this;
	}
	
	width_ {|width=0.1|
		netAddress.sendMsg("/iannix/setSize", id,  width);
		^this;	
	}
		
		

} 

